﻿<?xml version="1.0" encoding="UTF-8" ?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

  <xsl:template match="plan">
    <h1><xsl:value-of select="internal-name"/></h1>
    <xsl:apply-templates select="preconditions"/>
    <xsl:apply-templates select="invariants"/>
    <xsl:apply-templates select="postconditions"/>
    <xsl:apply-templates select="parameters"/>
    <xsl:apply-templates select="top-elements"/>
  </xsl:template>

  <xsl:template match="plan-element">
    <li>
      <xsl:if test="name != 'root'">
        <a>
          <xsl:choose>
            <xsl:when test="contains(name, 'sub')">
              <xsl:attribute name="href"><xsl:value-of select="name"/>.xml</xsl:attribute>
            </xsl:when>
            <xsl:otherwise>
              <xsl:attribute name="href">../actions/action/<xsl:value-of select="name"/>.xml</xsl:attribute>
            </xsl:otherwise>
          </xsl:choose>
          <xsl:value-of select="name"/>
        </a>
        <span>(</span>
        <xsl:for-each select="parameters/*">
          <span class="parameter"><xsl:value-of select="."/></span>
          <xsl:if test="position() != last()">
            <span>, </span>
          </xsl:if>
        </xsl:for-each>
        <span>)</span>
      </xsl:if>
      <ul>
        <xsl:apply-templates select="children"/>
      </ul>
    </li>
  </xsl:template>
  
  <xsl:template match="children">
    <xsl:apply-templates select="plan-element"/>
  </xsl:template>
  
  <xsl:template match="condition">
    <li>
      <xsl:choose>
        <xsl:when test="name = 'true'">
          <span class="true">true</span>
        </xsl:when>
        <xsl:when test="name = 'false'">
          <span class="false">false</span>
        </xsl:when>
        <xsl:when test="name = 'self'">
          <span class="prolog">self</span>
        </xsl:when>
        <xsl:when test="name = 'ball'">
          <span class="prolog">ball</span>
        </xsl:when>
        <xsl:when test="starts-with(name, '!')">
          !<a>
            <xsl:attribute name="href">../conditions/condition/<xsl:value-of select="substring-after(name, '!')"/>.xml</xsl:attribute>
            <xsl:value-of select="substring-after(name, '!')"/>
          </a>
        </xsl:when>
        <xsl:otherwise>
          <a>
            <xsl:attribute name="href">../conditions/condition/<xsl:value-of select="name"/>.xml</xsl:attribute>
            <xsl:value-of select="name"/>
          </a>
        </xsl:otherwise>
      </xsl:choose>
      <span>(</span>
      <xsl:for-each select="param">
        <span class="parameter"><xsl:value-of select="."/></span>
        <xsl:if test="position() != last()">
          <span>, </span>
        </xsl:if>
      </xsl:for-each>
      <span>)</span>
    </li>
  </xsl:template>

  <xsl:template match="preconditions">
    <div id="preconditions" class="section">
    <h2>Preconditions</h2>
    <ul>
      <xsl:apply-templates/>
    </ul>
    </div>
  </xsl:template>
  
  <xsl:template match="invariants">
    <div id="invariants" class="section">
    <h2>Invariants</h2>
    <ul>
      <xsl:apply-templates/>
    </ul>
    </div>
  </xsl:template>
  
  <xsl:template match="postconditions">
    <div id="postconditions" class="section">
    <h2>Postconditions</h2>
    <ul>
      <xsl:apply-templates/>
    </ul>
    </div>
  </xsl:template>

  <xsl:template match="parameters">
    <div id="parameters" class="section">
    <h2>Parameters</h2>
    <ul>
      <xsl:for-each select="*">
        <li><span class="parameter"><xsl:value-of select="."/></span> : <span class="type">&lt;<xsl:value-of select="name()"/>&gt;</span></li>
      </xsl:for-each>
    </ul>
    </div>
  </xsl:template>

  <xsl:template match="top-elements">
    <div id="top-elements" class="section">
    <h2>Plan</h2>
    <ul>
      <xsl:apply-templates/>
    </ul>
    </div>
  </xsl:template>
  
</xsl:stylesheet>
